#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <math.h>

#include <hpic.h>

int main(int argc, char *argv[])
{
  char ch = '-';
  size_t i, j, k;

  char infile[HPIC_STRNL];
  char outfile[HPIC_STRNL];
  size_t newnside = 0;
  int neworder = -1;
  int accum = 0;
  int writefull = 0;
  int writecut = 0;

  size_t nside;
  int order;
  int coord;
  int type;
  size_t nmaps;
  char command[HPIC_STRNL];
  char creator[HPIC_STRNL];
  char extname[HPIC_STRNL];
  char comment[HPIC_STRNL];
  hpic_int *pix = NULL;
  hpic_int *outpix = NULL;
  hpic_fltarr *maps = NULL;
  hpic_float *outdata = NULL;
  hpic_int *hits = NULL;
  hpic_int *outhits = NULL;
  hpic_float *errs = NULL;
  hpic_float *outerrs = NULL;
  hpic_keys *keys;
  hpic_float *tempmap;

  int temp;
  float val;

  if (argc == 1) {
    printf("\n");
    printf("<******************************************************>\n");
    printf("Usage:  hpic_convert -in <input FITS file>\n");
    printf("        -out <output FITS file>\n");
    printf("        -nside <new nside value>\n");
    printf("        -order <ordering (0=RING, 1=NEST)>\n");
    printf("        -accum (accumulate rather than average)\n");
    printf("        -full (write output to full-sphere file)\n");
    printf("        -cut (write output to cut-sphere file)\n");
    printf("<******************************************************>\n\n");
    exit(1);
  }

  /* parse command line */

  for (i = 0; i < argc; i++) {
    if (argv[i][0] == ch) {
      if (!strcmp(argv[i], "-in")) {
        strncpy(infile, argv[i + 1], HPIC_STRNL);
        i++;
      }
      if (!strcmp(argv[i], "-out")) {
        strncpy(outfile, argv[i + 1], HPIC_STRNL);
        i++;
      }
      if (!strcmp(argv[i], "-accum")) {
        accum = 1;
      }
      if (!strcmp(argv[i], "-full")) {
        writefull = 1;
      }
      if (!strcmp(argv[i], "-cut")) {
        writecut = 1;
      }
      if (!strcmp(argv[i], "-nside")) {
        newnside = (size_t) atoi(argv[i + 1]);
        i++;
      }
      if (!strcmp(argv[i], "-order")) {
        neworder = atoi(argv[i + 1]);
        if (neworder == 0) {
          neworder = HPIC_RING;
        } else {
          neworder = HPIC_NEST;
        }
        i++;
      }
    }
  }

  /* process FITS file */

  strcpy(comment, "");

  if (hpic_fits_map_test(infile, &nside, &order, &coord, &type, &nmaps)) {

    keys = hpic_keys_alloc();
    maps = hpic_fltarr_alloc(nmaps);
    for (i = 0; i < nmaps; i++) {
      tempmap = hpic_float_alloc(nside, order, coord, HPIC_AUTO);
      hpic_fltarr_set(maps, i, tempmap);
    }

    if (type == HPIC_FITS_FULL) {
      hpic_fits_full_read(infile, creator, extname, maps, keys);
      if (neworder != -1) {
        for (i = 0; i < nmaps; i++) {
          tempmap = hpic_fltarr_get(maps, i);
          if (neworder == HPIC_RING) {
            hpic_conv_float_ringcopy(tempmap);
          } else {
            hpic_conv_float_nestcopy(tempmap);
          }
        }
      }
      if (newnside != 0) {
        for (i = 0; i < nmaps; i++) {
          tempmap = hpic_fltarr_get(maps, i);
          if (accum) {
            outdata = hpic_conv_float_xgrade(tempmap, newnside, HPIC_CONV_ACCUM);
          } else {
            outdata = hpic_conv_float_xgrade(tempmap, newnside, HPIC_CONV_AVG);
          }
          hpic_float_free(tempmap);
          hpic_fltarr_set(maps, i, outdata);
        }
      }
      if (writecut) {
        outerrs = hpic_float_alloc(newnside, order, coord, HPIC_AUTO);
        outhits = hpic_int_alloc(newnside, order, coord, HPIC_AUTO);
        outpix = hpic_int_alloc(newnside, order, coord, HPIC_AUTO);
        for (i = 0; i < hpic_int_npix_get(outpix); i++) {
          if (!hpic_is_fnull(hpic_int_get(outhits, i))) {
            hpic_int_set(outpix, i, (int)i);
          }
        }
	hpic_fits_cut_write(outfile, "hpic_convert", "Converted Map", comment,
                          outpix, outhits, outerrs, maps, keys);
        
        hpic_int_free(outhits);
        hpic_float_free(outerrs);
        hpic_int_free(outpix);
      } else {
        hpic_fits_full_write(outfile, "hpic_convert", "Converted Map", comment,
                           maps, keys);
      }
    } else {
      errs = hpic_float_alloc(nside, order, coord, HPIC_AUTO);
      hits = hpic_int_alloc(nside, order, coord, HPIC_AUTO);
      pix = hpic_int_alloc(nside, order, coord, HPIC_AUTO);

      hpic_fits_cut_read(infile, creator, extname, pix, hits, errs, maps,
                         keys);

      if (neworder != -1) {
        if (neworder == HPIC_RING) {
          hpic_conv_float_ringcopy(errs);
          hpic_conv_int_ringcopy(hits);
          for (i = 0; i < nmaps; i++) {
            tempmap = hpic_fltarr_get(maps, i);
            hpic_conv_float_ringcopy(tempmap);
          }
        } else {
          hpic_conv_float_nestcopy(errs);
          hpic_conv_int_nestcopy(hits);
          for (i = 0; i < nmaps; i++) {
            tempmap = hpic_fltarr_get(maps, i);
            hpic_conv_float_nestcopy(tempmap);
          }
        }
      }
      if (newnside == 0) {
        outerrs = errs;
        outhits = hits;
        if (neworder != -1) {
          outpix =
            hpic_int_alloc(hpic_float_nside_get(errs),
                           hpic_float_order_get(errs),
                           hpic_float_coord_get(errs), HPIC_AUTO);
          for (i = 0; i < hpic_int_npix_get(outpix); i++) {
            if (!hpic_is_inull(hpic_int_get(outhits, i))) {
              hpic_int_set(outpix, i, (int)i);
            }
          }
        } else {
          outpix = pix;
        }
      } else {
        for (i = 0; i < nmaps; i++) {
          tempmap = hpic_fltarr_get(maps, i);
          if (accum) {
            outdata = hpic_conv_float_xgrade(tempmap, newnside, HPIC_CONV_ACCUM);
          } else {
            outdata = hpic_conv_float_xgrade(tempmap, newnside, HPIC_CONV_AVG);
          }
          hpic_float_free(tempmap);
          hpic_fltarr_set(maps, i, outdata);
        }
        if (accum) {
          outerrs = hpic_conv_float_xgrade(errs, newnside, HPIC_CONV_ACCUM);
        } else {
          outerrs = hpic_conv_float_xgrade(errs, newnside, HPIC_CONV_AVG);
        }
        outhits = hpic_conv_int_xgrade(hits, newnside, HPIC_CONV_ACCUM);
        
        outpix =
          hpic_int_alloc(newnside, hpic_float_order_get(errs),
                         hpic_float_coord_get(errs), HPIC_AUTO);
        for (i = 0; i < hpic_int_npix_get(outhits); i++) {
          if (!hpic_is_inull(hpic_int_get(outhits, i))) {
            hpic_int_set(outpix, i, (int)i);
          }
        }
      }

      if (writecut) {
	hpic_fits_cut_write(outfile, "hpic_convert", "Converted Map", comment,
			    outpix, outhits, outerrs, maps, keys);
      } else {
        hpic_fits_full_write(outfile, "hpic_convert", "Converted Map", comment,
                             maps, keys);
      }

      hpic_int_free(pix);
      hpic_int_free(hits);
      hpic_float_free(errs);
      if (newnside != 0) {
        hpic_int_free(outpix);
        hpic_int_free(outhits);
        hpic_float_free(outerrs);
      } else {
        if (neworder != -1) {
          hpic_int_free(outpix);
        }
      }
    }

    hpic_keys_free(keys);
    for (i = 0; i < nmaps; i++) {
      tempmap = hpic_fltarr_get(maps, i);
      hpic_float_free(tempmap);
    }
    hpic_fltarr_free(maps);

  } else {
    fprintf(stderr, "hpic_convert:  FITS file is not a recognized type\n");
    exit(1);
  }

  return 0;
}
